defmodule WhiteBreadContext.Courier do
  use WhiteBread.Context
  use Hound.Helpers
  alias Volt.{Repo, Accounts.Courier}

  feature_starting_state fn  ->
    Application.ensure_all_started(:hound)
    %{}
  end
  scenario_starting_state fn _state ->
    Hound.start_session
    Ecto.Adapters.SQL.Sandbox.checkout(Volt.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(Volt.Repo, {:shared, self()})

    %{}
  end
  scenario_finalize fn _status, _state ->
    Hound.end_session
    Ecto.Adapters.SQL.Sandbox.checkin(Volt.Repo)
  end

  given_ ~r/^that no user is logged in$/, fn state ->
    {:ok, state}
  end

  and_ ~r/^my email is "(?<email>[^"]+)"$/,
  fn state, %{email: email} ->
    {:ok, state |> Map.put(:email, email)}
  end

  and_ ~r/^my password is "(?<password>[^"]+)"$/,
  fn state, %{password: password} ->
    {:ok, state |> Map.put(:password, password)}
  end

  and_ ~r/^my name is "(?<first_name>[^"]+)", "(?<last_name>[^"]+)"$/,
  fn state, %{first_name: first_name, last_name: last_name} ->
    {:ok, state |> Map.put(:first_name, first_name) |> Map.put(:last_name, last_name)}
  end

  and_ ~r/^my phone number is "(?<phone_number>[^"]+)"$/,
  fn state, %{phone_number: phone_number} ->
    {:ok, state |> Map.put(:phone_number, phone_number)}
  end

  and_ ~r/^my status is "(?<courier_status>[^"]+)"$/,
  fn state, %{courier_status: courier_status} ->
    {:ok, state |> Map.put(:courier_status, courier_status)}
  end

  and_ ~r/^I open the welcome page$/, fn state ->
    navigate_to "/"
    {:ok, state}
  end

  and_ ~r/^I click the register as a courier button$/, fn state ->
    click({:id, "courier_register"})
    {:ok, state}
  end

  and_ ~r/^I enter my information$/, fn state ->
    fill_field({:id, "courier_email"}, state[:email])
    fill_field({:id, "courier_password"}, state[:password])
    fill_field({:id, "courier_first_name"}, state[:first_name])
    fill_field({:id, "courier_last_name"}, state[:last_name])
    fill_field({:id, "courier_phone_number"}, state[:phone_number])
    {:ok, state}
  end

  when_ ~r/^I submit the form$/, fn state ->
    click({:id, "submit_button"})
    {:ok, state}
  end

  then_ ~r/^I should receive a confirmation message$/, fn state ->
    assert visible_in_page? ~r/Courier created successfully./
    {:ok, state}
  end

  and_ ~r/^the following courier already exist$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn courier -> Courier.changeset(%Courier{}, courier) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)
    {:ok, state}
  end
end
