defmodule WhiteBreadContext.Customer do
  use WhiteBread.Context
  use Hound.Helpers
  use VoltWeb, :controller
  import Volt.AccountsFixtures
  alias Volt.Sales.Item
  alias Volt.Repo

  feature_starting_state fn  ->
    Application.ensure_all_started(:hound)
    %{}
  end
  scenario_starting_state fn _state ->
    Hound.start_session
    Ecto.Adapters.SQL.Sandbox.checkout(Volt.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(Volt.Repo, {:shared, self()})

    %{}
  end
  scenario_finalize fn _status, _state ->
    Hound.end_session
    Ecto.Adapters.SQL.Sandbox.checkin(Volt.Repo)
  end

  given_ ~r/^that no user is logged in$/, fn state ->
    {:ok, state}
  end

  and_ ~r/^I log in$/, fn state ->
    customer = customer_fixture(%{
      address: "Aida 11",
      balance: 69.0,
      birth_date: ~D[2000-11-04],
      card_number: "123123123123123",
      email: "heyDavid@gmail.com",
      password: "areyoureadingthis123456",
      first_name: "Because",
      last_name: "Damnyourhandsome",
      phone_number: "+33641941064",
      city: "Tartu",
      zip_code: "51009"
    })
    navigate_to "/customers/log_in"
    fill_field({:id,"customer_email"}, "heyDavid@gmail.com")
    fill_field({:id,"customer_password"}, "areyoureadingthis123456")
    click({:id, "submit_button"})
    assert visible_in_page? ~r/Listing restaurants/
    {:ok, state}
  end

  given_ ~r/^that restaurant "(?<restaurant_name>[^"]+)" with item "(?<item_name>[^"]+)" and price of "(?<item_price>[^"]+)" exists$/,
  fn state, %{restaurant_name: restaurant_name, item_name: item_name, item_price: item_price} ->
    restaurant = restaurant_fixture(%{
      address: "Kastani 42",
      closing_time: ~T[14:00:00],
      email: "krevetki@polska.pl",
      first_name: "Pjotr",
      last_name: "Czeckej",
      name: restaurant_name,
      opening_time: ~T[14:00:00],
      password: "kurwakurwakurwa123",
      phone_number: "+33641941068",
      city: "Tartu",
      zip_code: "50410",
      price_level: 10
    })
    item = %Item{name: item_name, unit_price: item_price, description: "Best pelmen in city"}
    item = Ecto.build_assoc(restaurant, :items, item) |> Repo.insert!()
    {:ok, state |> Map.put(:restaurant_name, restaurant_name) |> Map.put(:item_name, item_name) |> Map.put(:item_price, item_price)}
  end

  and_ ~r/^I navigate to restaurant's menu$/, fn state ->
    :timer.sleep(2000)
    click({:id, "order_button"})
    assert visible_in_page? ~r/Polish Dream/
    {:ok, state}
  end

  and_ ~r/^I choose an item that I want to order$/, fn state ->
    click({:id, "order_button"})
    assert visible_in_page? ~r/Pelmeni/
    {:ok, state}
  end

  when_ ~r/^I submit the item button$/, fn state ->
    click({:id, "submit"})
    {:ok, state}
  end

  then_ ~r/^I should receive an updated basket confirmation$/, fn state ->
    assert visible_in_page? ~r/Basket updated!/
    {:ok, state}
  end

  when_ ~r/^I got logged in$/, fn state ->
    assert visible_in_page? ~r/Greetings,/
    assert visible_in_page? ~r/Listing restaurants/
    {:ok, state}
  end

  then_ ~r/^I should see list of restaurants$/, fn state ->
    assert visible_in_page? ~r/Polish Dream/
    assert visible_in_page? ~r/Order/
    {:ok, state}
  end

  and_ ~r/^I navigate to basket$/, fn state ->
    click({:id, "basket_button"})
    assert visible_in_page? ~r/Order total: 7.5 €/
    {:ok, state}
  end

  when_ ~r/^I submit the order button$/, fn state ->
    click({:id, "basket_submit"})
    {:ok, state}
  end

  when_ ~r/^I remove the item from basket$/, fn state ->
    click({:id, "item_delete"})
    {:ok, state}
  end

  then_ ~r/^order total should change$/, fn state ->
    assert visible_in_page? ~r/Order total: 0.0 €/
    {:ok, state}
  end

  then_ ~r/^I should receive an order confirmation$/, fn state ->
    assert visible_in_page? ~r/Your order has been submitted!/
    assert visible_in_page? ~r/Order total: 0.0 €/
    {:ok, state}
  end

  and_ ~r/^my email is "(?<email>[^"]+)" and password is "(?<password>[^"]+)"$/,
  fn state, %{email: email, password: password} ->
    {:ok, state |> Map.put(:email, email) |> Map.put(:password, password)}
  end

  and_ ~r/^my name is "(?<first_name>[^"]+)", "(?<last_name>[^"]+)"$/,
  fn state, %{first_name: first_name, last_name: last_name} ->
    {:ok, state |> Map.put(:first_name, first_name) |> Map.put(:last_name, last_name)}
  end

  and_ ~r/^my phone number is "(?<phone_number>[^"]+)"$/,
  fn state, %{phone_number: phone_number} ->
    {:ok, state |> Map.put(:phone_number, phone_number)}
  end

  and_ ~r/^my address is "(?<address>[^"]+)", "(?<zip_code>[^"]+)", "(?<city>[^"]+)"$/,
  fn state, %{address: address, zip_code: zip_code, city: city} ->
    {:ok, state |> Map.put(:address, address) |> Map.put(:zip_code, zip_code) |> Map.put(:city, city)}
  end

  and_ ~r/^my birth date is "(?<birth_date>[^"]+)"$/,
  fn state, %{birth_date: birth_date} ->
    {:ok, state |> Map.put(:birth_date, birth_date)}
  end

  and_ ~r/^my bank card is "(?<card_number>[^"]+)"$/,
  fn state, %{card_number: card_number} ->
    {:ok, state |> Map.put(:card_number, card_number)}
  end

  and_ ~r/^I open the welcome page$/, fn state ->
    navigate_to "/"
    {:ok, state}
  end

  and_ ~r/^I navigate to the deposit menu$/, fn state ->
    :timer.sleep(1000)
    click({:id, "deposit_button"})
    assert visible_in_page? ~r/Add to balance/
    {:ok, state}
  end

  and_ ~r/^I fill in the deposit amount of "(?<balance>[^"]+)"$/, fn state, %{balance: balance} ->
    fill_field({:id, "balance"}, balance)
    {:ok, state}
  end

  when_ ~r/^I click the deposit button$/, fn state ->
    click({:id, "deposit_submit"})
    {:ok, state}
  end

  then_ ~r/^I should receive a deposit confirmation$/, fn state ->
    assert visible_in_page? ~r/Your balance has been updated!/
    {:ok, state}
  end

  then_ ~r/^I should receive a deposit rejection$/, fn state ->
    assert visible_in_page? ~r/Deposit too small! Just like something else of yours!/
    {:ok, state}
  end

  and_ ~r/^I click the register as customer button$/, fn state ->
    click({:id, "customer_register"})
    {:ok, state}
  end

  and_ ~r/^I enter my information$/, fn state ->
    fill_field({:id, "email"}, state[:email])
    fill_field({:id, "password"}, state[:password])
    fill_field({:id, "first_name"}, state[:first_name])
    fill_field({:id, "last_name"}, state[:last_name])
    fill_field({:id, "phone_number"}, state[:phone_number])
    # fill_field({:id, "birth_date"}, state[:birth_date])
    fill_field({:id, "address"}, state[:address])
    fill_field({:id, "zip_code"}, state[:zip_code])
    fill_field({:id, "city"}, state[:city])
    fill_field({:id, "card_number"}, state[:card_number])
    {:ok, state}
  end

  when_ ~r/^I summit the register button$/, fn state ->
    click({:id, "submit_button"})
    {:ok, state}
  end

  then_ ~r/^I should receive a confirmation message$/, fn state ->
    assert visible_in_page? ~r/Customer created successfully./
    {:ok, state}
  end

  then_ ~r/^I should receive a rejection message$/, fn state ->
    assert visible_in_page? ~r/Oops, something went wrong! Please check the errors below./
    {:ok, state}
  end

  and_ ~r/^the following customer already exist$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn customer -> customer_fixture(customer) end)
    {:ok, state}
  end

  # use current location/Map

  when_ ~r/^I click on `use current location` checkbox$/, fn state ->
    click({:id, "use_current_location_check"})
    {:ok, state}
  end

  then_ ~r/^Address, city and postal code fields are filled$/, fn state ->
    assert inner_text({:id, "city"}) !=  nil
    assert inner_text({:id, "address"}) !=  nil
    assert inner_text({:id, "zip_code"}) !=  nil
    {:ok, state}
  end
end
