defmodule Volt.Sales.Item do
  use Ecto.Schema
  import Ecto.Changeset
  alias Volt.Sales

  schema "items" do
    field :description, :string
    field :name, :string
    field :unit_price, :string
    field :orderable, :boolean, default: true
    field :discount, :integer, default: 0
    belongs_to :restaurant, Volt.Accounts.Restaurant
    has_many :basket, Sales.Basket

    timestamps()
  end

  def changeset(item, attrs) do
    item
    |> cast(attrs, [:name, :description, :unit_price, :orderable, :discount])
    |> validate_required([:name, :unit_price, :orderable, :discount])
    |> validate_inclusion(:discount, 0..100)
  end
end
