defmodule Volt.Sales.Order do
  use Ecto.Schema
  import Ecto.Changeset
  alias Volt.Accounts
  alias Volt.Sales
  alias Volt.Repo

  schema "orders" do
    field :delivery_price, :float
    field :delivery_duration, :integer
    field :delivery_time, :naive_datetime
    field :order_price, :float
    field :order_status, :string, default: "PENDING"
    field :preparation_duration, :integer
    field :preparation_time, :naive_datetime
    field :restaurant_message, :string
    field :total_price, :float
    belongs_to :customer, Accounts.Customer
    belongs_to :restaurant, Accounts.Restaurant
    belongs_to :courier, Accounts.Courier
    has_many :basket, Sales.Basket

    timestamps()
  end

  @doc false
  def changeset(order, attrs) do
    order
    |> cast(attrs, [:order_status, :preparation_duration, :preparation_time, :delivery_duration, :delivery_time, :delivery_price, :restaurant_message, :order_price, :total_price, :courier_id])
    |> validate_required([:order_status, :order_price])
    |> validate_inclusion(:preparation_duration, 1..61)
    |> validate_prep()
    |> validate_delivery()
  end

  defp validate_prep(changeset) do
    submition = get_field(changeset, :inserted_at)
    prep = get_field(changeset, :preparation_time)

    if(prep != nil and prep < submition) do
      add_error(changeset, :preparation_time, "Preparation time must be after submition!")
    else
      changeset
    end
  end

  defp validate_delivery(changeset) do
    prep = get_field(changeset, :preparation_time)
    delivery = get_field(changeset, :delivery_time)

    if(delivery != nil and prep != nil and delivery < prep) do
      add_error(changeset, :delivery_time, "Delivery time must be after preparation !")
    else
      changeset
    end
  end
end
