defmodule VoltWeb.CourierDashboardController do
  use VoltWeb, :controller
  import Ecto.Query, only: [from: 2]
  import Ecto.Changeset
  alias Volt.Sales.Basket
  alias Volt.{Repo, Accounts}
  alias VoltWeb.RestaurantAuth
  alias Volt.Sales.Order
  alias Volt.Sales
  alias Volt.Accounts.Courier
  alias VoltWeb.CourierAuth
  require Logger

  plug :assign_changeset

  defp orders_for_delivery() do
    orders =
      from(o in Order, where: (o.order_status == ^"IN_PROCESS" or o.order_status == ^"PREPARED") and is_nil(o.courier_id))
      |> Repo.all()
      |> Repo.preload([:restaurant, :customer])
    orders
  end

  defp orders_being_delivered(courier) do
    orders =
      from(o in Order, where: o.courier_id == ^courier.id and o.order_status != ^"DELIVERED")
      |> Repo.all()
      |> Repo.preload([:restaurant, :customer, :courier])
    orders
  end

  def index(conn, _params) do
    cour = conn.assigns.current_courier
    render(conn, "index.html", orders_for_delivery: orders_for_delivery(), courier: cour, orders_being_delivered: orders_being_delivered(cour))
  end

  def update(conn, %{"action" => "update_status"} = params) do
    %{"current_password" => password, "courier" => courier_params} = params
    courier = conn.assigns.current_courier

    case Accounts.update_courier_status(courier, password, courier_params) do
      {:ok, courier} ->
        conn
        |> put_flash(:info, "Status updated successfully.")
        |> put_session(:courier_return_to, Routes.courier_dashboard_path(conn, :index))
        |> CourierAuth.log_in_courier(courier)

      {:error, changeset} ->
        render(conn, "index.html", status_changeset: changeset)
    end
  end

  def accept(conn, %{"id" => id}) do
    courier = conn.assigns.current_courier

    order =
      from(o in Order, where: o.id == ^id)
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer])


    courier =
      from(cour in Courier, where: cour.id == ^courier.id)
      |> Repo.one()
      |> Repo.preload([:orders])



    order |> Repo.preload([:courier, :restaurant, :customer]) |> Order.changeset(%{}) |> Ecto.Changeset.put_assoc(:courier, courier) |> Repo.update!()

    conn
    |> put_flash(:info, "Order marked as yours successfully.")
    |> redirect(to: Routes.courier_dashboard_path(conn, :index))


  end

  def close_delivery(conn, %{"id" => id}) do
    courier = conn.assigns.current_courier

    order =
      from(o in Order, where: o.id == ^id)
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer])

    case Sales.update_order(order, Map.from_struct(%{order | courier: courier.id, order_status: "DELIVERED"})) do
      {:ok, _order} ->
        conn
        |> put_flash(:info, "Order marked as yours successfully.")
        |> redirect(to: Routes.courier_dashboard_path(conn, :index))
      {:error, %Ecto.Changeset{} = _changeset} ->
        conn
        |> put_flash(:info, "Error ! Couldn't mark order for you.")
        |> redirect(to: Routes.courier_dashboard_path(conn, :index))

    end
  end

  defp assign_changeset(conn, _opts) do
    courier = conn.assigns.current_courier
    conn
    |> assign(:status_changeset, Accounts.change_courier_status(courier))
  end
end
