defmodule VoltWeb.CourierResetPasswordController do
  use VoltWeb, :controller

  alias Volt.Accounts

  plug :get_courier_by_reset_password_token when action in [:edit, :update]

  def new(conn, _params) do
    render(conn, "new.html")
  end

  def create(conn, %{"courier" => %{"email" => email}}) do
    if courier = Accounts.get_courier_by_email(email) do
      Accounts.deliver_courier_reset_password_instructions(
        courier,
        &Routes.courier_reset_password_url(conn, :edit, &1)
      )
    end

    conn
    |> put_flash(
      :info,
      "If your email is in our system, you will receive instructions to reset your password shortly."
    )
    |> redirect(to: "/")
  end

  def edit(conn, _params) do
    render(conn, "edit.html", changeset: Accounts.change_courier_password(conn.assigns.courier))
  end

  # Do not log in the courier after reset password to avoid a
  # leaked token giving the courier access to the account.
  def update(conn, %{"courier" => courier_params}) do
    case Accounts.reset_courier_password(conn.assigns.courier, courier_params) do
      {:ok, _} ->
        conn
        |> put_flash(:info, "Password reset successfully.")
        |> redirect(to: Routes.courier_session_path(conn, :new))

      {:error, changeset} ->
        render(conn, "edit.html", changeset: changeset)
    end
  end

  defp get_courier_by_reset_password_token(conn, _opts) do
    %{"token" => token} = conn.params

    if courier = Accounts.get_courier_by_reset_password_token(token) do
      conn |> assign(:courier, courier) |> assign(:token, token)
    else
      conn
      |> put_flash(:error, "Reset password link is invalid or it has expired.")
      |> redirect(to: "/")
      |> halt()
    end
  end
end
