defmodule VoltWeb.CourierSettingsController do
  use VoltWeb, :controller

  alias Volt.Accounts
  alias VoltWeb.CourierAuth

  plug :assign_changesets

  def edit(conn, _params) do
    render(conn, "edit.html")
  end

  def update(conn, %{"action" => "update_email"} = params) do
    %{"current_password" => password, "courier" => courier_params} = params
    courier = conn.assigns.current_courier

    case Accounts.apply_courier_email(courier, password, courier_params) do
      {:ok, applied_courier} ->
        Accounts.deliver_courier_update_email_instructions(
          applied_courier,
          courier.email,
          &Routes.courier_settings_url(conn, :confirm_email, &1)
        )

        conn
        |> put_flash(
          :info,
          "A link to confirm your email change has been sent to the new address."
        )
        |> redirect(to: Routes.courier_settings_path(conn, :edit))

      {:error, changeset} ->
        render(conn, "edit.html", email_changeset: changeset)
    end
  end

  def update(conn, %{"action" => "update_password"} = params) do
    %{"current_password" => password, "courier" => courier_params} = params
    courier = conn.assigns.current_courier

    case Accounts.update_courier_password(courier, password, courier_params) do
      {:ok, courier} ->
        conn
        |> put_flash(:info, "Password updated successfully.")
        |> put_session(:courier_return_to, Routes.courier_settings_path(conn, :edit))
        |> CourierAuth.log_in_courier(courier)

      {:error, changeset} ->
        render(conn, "edit.html", password_changeset: changeset)
    end
  end

  def update(conn, %{"action" => "update_profile"} = params) do
    %{"current_password" => password, "courier" => courier_params} = params
    courier = conn.assigns.current_courier

    case Accounts.update_courier_profile(courier, password, courier_params) do
      {:ok, courier} ->
        conn
        |> put_flash(:info, "Profile updated successfully.")
        |> put_session(:courier_return_to, Routes.courier_settings_path(conn, :edit))
        |> CourierAuth.log_in_courier(courier)

      {:error, changeset} ->
        render(conn, "edit.html", profile_changeset: changeset)
    end
  end

  def confirm_email(conn, %{"token" => token}) do
    case Accounts.update_courier_email(conn.assigns.current_courier, token) do
      :ok ->
        conn
        |> put_flash(:info, "Email changed successfully.")
        |> redirect(to: Routes.courier_settings_path(conn, :edit))

      :error ->
        conn
        |> put_flash(:error, "Email change link is invalid or it has expired.")
        |> redirect(to: Routes.courier_settings_path(conn, :edit))
    end
  end

  defp assign_changesets(conn, _opts) do
    courier = conn.assigns.current_courier

    conn
    |> assign(:email_changeset, Accounts.change_courier_email(courier))
    |> assign(:password_changeset, Accounts.change_courier_password(courier))
    |> assign(:profile_changeset, Accounts.change_courier_profile(courier))
  end
end
