defmodule VoltWeb.CustomerRegistrationController do
  use VoltWeb, :controller

  alias Volt.Accounts
  alias Volt.Accounts.Customer
  alias VoltWeb.CustomerAuth

  def new(conn, _params) do
    changeset = Accounts.change_customer_registration(%Customer{})
    render(conn, "new.html", changeset: changeset, bing_map: get_map())
  end

  def create(conn, %{"customer" => customer_params}) do
    case Accounts.register_customer(customer_params) do
      {:ok, customer} ->
        {:ok, _} =
          Accounts.deliver_customer_confirmation_instructions(
            customer,
            &Routes.customer_confirmation_url(conn, :edit, &1)
          )

        conn
        |> put_flash(:info, "Customer created successfully.")
        |> CustomerAuth.log_in_customer(customer)

      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, "new.html", changeset: changeset, bing_map: get_map())
    end
  end

  defp get_map(), do: "https://www.bing.com/api/maps/mapcontrol?callback=GetMap&key=" <> System.get_env("BING_MAP_KEY")
end
