defmodule VoltWeb.CustomerSettingsController do
  use VoltWeb, :controller

  alias Volt.Accounts
  alias VoltWeb.CustomerAuth

  plug :assign_changesets

  def edit(conn, _params) do
    render(conn, "edit.html")
  end

  def update(conn, %{"action" => "update_email"} = params) do
    %{"current_password" => password, "customer" => customer_params} = params
    customer = conn.assigns.current_customer

    case Accounts.apply_customer_email(customer, password, customer_params) do
      {:ok, applied_customer} ->
        Accounts.deliver_customer_update_email_instructions(
          applied_customer,
          customer.email,
          &Routes.customer_settings_url(conn, :confirm_email, &1)
        )

        conn
        |> put_flash(
          :info,
          "A link to confirm your email change has been sent to the new address."
        )
        |> redirect(to: Routes.customer_settings_path(conn, :edit))

      {:error, changeset} ->
        render(conn, "edit.html", email_changeset: changeset)
    end
  end

  def update(conn, %{"action" => "update_password"} = params) do
    %{"current_password" => password, "customer" => customer_params} = params
    customer = conn.assigns.current_customer

    case Accounts.update_customer_password(customer, password, customer_params) do
      {:ok, customer} ->
        conn
        |> put_flash(:info, "Password updated successfully.")
        |> put_session(:customer_return_to, Routes.customer_settings_path(conn, :edit))
        |> CustomerAuth.log_in_customer(customer)

      {:error, changeset} ->
        render(conn, "edit.html", password_changeset: changeset)
    end
  end

  def update(conn, %{"action" => "update_profile"} = params) do
    %{"current_password" => password, "customer" => customer_params} = params
    customer = conn.assigns.current_customer

    case Accounts.update_customer_profile(customer, password, customer_params) do
      {:ok, customer} ->
        conn
        |> put_flash(:info, "Profile updated successfully.")
        |> put_session(:customer_return_to, Routes.customer_settings_path(conn, :edit))
        |> CustomerAuth.log_in_customer(customer)

      {:error, changeset} ->
        render(conn, "edit.html", profile_changeset: changeset)
    end
  end

  def confirm_email(conn, %{"token" => token}) do
    case Accounts.update_customer_email(conn.assigns.current_customer, token) do
      :ok ->
        conn
        |> put_flash(:info, "Email changed successfully.")
        |> redirect(to: Routes.customer_settings_path(conn, :edit))

      :error ->
        conn
        |> put_flash(:error, "Email change link is invalid or it has expired.")
        |> redirect(to: Routes.customer_settings_path(conn, :edit))
    end
  end

  defp assign_changesets(conn, _opts) do
    customer = conn.assigns.current_customer

    conn
    |> assign(:email_changeset, Accounts.change_customer_email(customer))
    |> assign(:password_changeset, Accounts.change_customer_password(customer))
    |> assign(:profile_changeset, Accounts.change_customer_profile(customer))
  end
end
