defmodule VoltWeb.RestaurantItemController do
  use VoltWeb, :controller
  require Logger
  import Ecto.Query, only: [from: 2]

  alias Volt.Sales
  alias Volt.Sales.Item
  alias Volt.Repo

  def index(conn, _params) do
    restaurant = conn.assigns.current_restaurant
    items =
      from(i in Item, where: i.restaurant_id == ^restaurant.id and i.orderable == ^true)
      |> Repo.all()
      |> Repo.preload(:restaurant)
    render(conn, "index.html", items: items)
  end

  def new(conn, _params) do
    changeset = Sales.change_item(%Item{})
    render(conn, "new.html", changeset: changeset)
  end

  def create(conn, %{"item" => item_params}) do
    restaurant = conn.assigns.current_restaurant
    item_params = %{item_params | "discount" => String.to_integer(item_params["discount"])}
    item_struct = Ecto.build_assoc(restaurant, :items, Enum.map(item_params, fn({key, value}) -> {String.to_atom(key), value} end))
    changeset = Item.changeset(item_struct, %{})
    case Repo.insert(changeset) do
      {:ok, item} ->
        conn
        |> put_flash(:info, "Item created successfully.")
        |> redirect(to: Routes.restaurant_item_path(conn, :show, item))

      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, "new.html", changeset: changeset)
    end
  end

  def show(conn, %{"id" => id}) do
    item =
      from(i in Item, where: i.id == ^id and i.orderable == ^true)
      |> Repo.one()
      |> Repo.preload(:restaurant)
    render(conn, "show.html", item: item)
  end

  def edit(conn, %{"id" => id}) do
    item =
      from(i in Item, where: i.id == ^id and i.orderable == ^true)
      |> Repo.one()
      |> Repo.preload(:restaurant)
    changeset = Sales.change_item(item)
    render(conn, "edit.html", item: item, changeset: changeset)
  end

  def update(conn, %{"id" => id, "item" => item_params}) do
    item =
      from(i in Item, where: i.id == ^id and i.orderable == ^true)
      |> Repo.one()
      |> Repo.preload(:restaurant)

    case Sales.update_item(item, item_params) do
      {:ok, item} ->
        conn
        |> put_flash(:info, "Item updated successfully.")
        |> redirect(to: Routes.restaurant_item_path(conn, :show, item))

      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, "edit.html", item: item, changeset: changeset)
    end
  end

  def delete(conn, %{"id" => id}) do
    item =
      from(i in Item, where: i.id == ^id and i.orderable == ^true)
      |> Repo.one()
      |> Repo.preload(:restaurant)

    {:ok, _item} = Sales.update_item(item, %{orderable: false, discount: 0})

    conn
    |> put_flash(:info, "Item deleted successfully.")
    |> redirect(to: Routes.restaurant_item_path(conn, :index))
  end
end
