defmodule VoltWeb.RestaurantResetPasswordController do
  use VoltWeb, :controller

  alias Volt.Accounts

  plug :get_restaurant_by_reset_password_token when action in [:edit, :update]

  def new(conn, _params) do
    render(conn, "new.html")
  end

  def create(conn, %{"restaurant" => %{"email" => email}}) do
    if restaurant = Accounts.get_restaurant_by_email(email) do
      Accounts.deliver_restaurant_reset_password_instructions(
        restaurant,
        &Routes.restaurant_reset_password_url(conn, :edit, &1)
      )
    end

    conn
    |> put_flash(
      :info,
      "If your email is in our system, you will receive instructions to reset your password shortly."
    )
    |> redirect(to: "/")
  end

  def edit(conn, _params) do
    render(conn, "edit.html", changeset: Accounts.change_restaurant_password(conn.assigns.restaurant))
  end

  # Do not log in the restaurant after reset password to avoid a
  # leaked token giving the restaurant access to the account.
  def update(conn, %{"restaurant" => restaurant_params}) do
    case Accounts.reset_restaurant_password(conn.assigns.restaurant, restaurant_params) do
      {:ok, _} ->
        conn
        |> put_flash(:info, "Password reset successfully.")
        |> redirect(to: Routes.restaurant_session_path(conn, :new))

      {:error, changeset} ->
        render(conn, "edit.html", changeset: changeset)
    end
  end

  defp get_restaurant_by_reset_password_token(conn, _opts) do
    %{"token" => token} = conn.params

    if restaurant = Accounts.get_restaurant_by_reset_password_token(token) do
      conn |> assign(:restaurant, restaurant) |> assign(:token, token)
    else
      conn
      |> put_flash(:error, "Reset password link is invalid or it has expired.")
      |> redirect(to: "/")
      |> halt()
    end
  end
end
