defmodule VoltWeb.RestaurantSettingsController do
  use VoltWeb, :controller

  alias Volt.Accounts
  alias VoltWeb.RestaurantAuth

  plug :assign_changesets

  def edit(conn, _params) do
    render(conn, "edit.html")
  end

  def update(conn, %{"action" => "update_email"} = params) do
    %{"current_password" => password, "restaurant" => restaurant_params} = params
    restaurant = conn.assigns.current_restaurant

    case Accounts.apply_restaurant_email(restaurant, password, restaurant_params) do
      {:ok, applied_restaurant} ->
        Accounts.deliver_restaurant_update_email_instructions(
          applied_restaurant,
          restaurant.email,
          &Routes.restaurant_settings_url(conn, :confirm_email, &1)
        )

        conn
        |> put_flash(
          :info,
          "A link to confirm your email change has been sent to the new address."
        )
        |> redirect(to: Routes.restaurant_settings_path(conn, :edit))

      {:error, changeset} ->
        render(conn, "edit.html", email_changeset: changeset)
    end
  end

  def update(conn, %{"action" => "update_password"} = params) do
    %{"current_password" => password, "restaurant" => restaurant_params} = params
    restaurant = conn.assigns.current_restaurant

    case Accounts.update_restaurant_password(restaurant, password, restaurant_params) do
      {:ok, restaurant} ->
        conn
        |> put_flash(:info, "Password updated successfully.")
        |> put_session(:restaurant_return_to, Routes.restaurant_settings_path(conn, :edit))
        |> RestaurantAuth.log_in_restaurant(restaurant)

      {:error, changeset} ->
        render(conn, "edit.html", password_changeset: changeset)
    end
  end

  def update(conn, %{"action" => "update_profile"} = params) do
    %{"current_password" => password, "restaurant" => restaurant_params} = params
    restaurant = conn.assigns.current_restaurant

    case Accounts.update_restaurant_profile(restaurant, password, restaurant_params) do
      {:ok, restaurant} ->
        conn
        |> put_flash(:info, "Profile updated successfully.")
        |> put_session(:restaurant_return_to, Routes.restaurant_settings_path(conn, :edit))
        |> RestaurantAuth.log_in_restaurant(restaurant)

      {:error, changeset} ->
        render(conn, "edit.html", profile_changeset: changeset)
    end
  end


  def confirm_email(conn, %{"token" => token}) do
    case Accounts.update_restaurant_email(conn.assigns.current_restaurant, token) do
      :ok ->
        conn
        |> put_flash(:info, "Email changed successfully.")
        |> redirect(to: Routes.restaurant_settings_path(conn, :edit))

      :error ->
        conn
        |> put_flash(:error, "Email change link is invalid or it has expired.")
        |> redirect(to: Routes.restaurant_settings_path(conn, :edit))
    end
  end

  defp assign_changesets(conn, _opts) do
    restaurant = conn.assigns.current_restaurant

    conn
    |> assign(:email_changeset, Accounts.change_restaurant_email(restaurant))
    |> assign(:password_changeset, Accounts.change_restaurant_password(restaurant))
    |> assign(:profile_changeset, Accounts.change_restaurant_profile(restaurant))
  end
end
