defmodule VoltWeb.Router do


  use VoltWeb, :router

  import VoltWeb.CustomerAuth

  import VoltWeb.RestaurantAuth

  import VoltWeb.CourierAuth

  pipeline :browser do
    plug :accepts, ["html"]
    plug :fetch_session
    plug :fetch_live_flash
    plug :put_root_layout, {VoltWeb.LayoutView, :root}
    plug :protect_from_forgery
    plug :put_secure_browser_headers
    plug :fetch_current_customer
    plug :fetch_current_restaurant
    plug :fetch_current_courier
    plug NavigationHistory.Tracker
  end

  pipeline :api do
    plug :accepts, ["json"]
    plug NavigationHistory.Tracker
  end

  scope "/", VoltWeb do
    pipe_through :browser
    get "/", PageController, :index
  end



  # Other scopes may use custom stacks.
  # scope "/api", VoltWeb do
  #   pipe_through :api
  # end

  # Enables LiveDashboard only for development
  #
  # If you want to use the LiveDashboard in production, you should put
  # it behind authentication and allow only admins to access it.
  # If your application does not have an admins-only section yet,
  # you can use Plug.BasicAuth to set up some basic authentication
  # as long as you are also using SSL (which you should anyway).
  if Mix.env() in [:dev, :test] do
    import Phoenix.LiveDashboard.Router

    scope "/" do
      pipe_through :browser
      live_dashboard "/dashboard", metrics: VoltWeb.Telemetry
    end
  end

  # Enables the Swoosh mailbox preview in development.
  #
  # Note that preview only shows emails that were sent by the same
  # node running the Phoenix server.
  if Mix.env() == :dev do
    scope "/dev" do
      pipe_through :browser

      forward "/mailbox", Plug.Swoosh.MailboxPreview
    end
  end



  ## Authentication routes

  scope "/", VoltWeb do
    pipe_through [:browser, :redirect_if_courier_is_authenticated]

    get "/couriers/register", CourierRegistrationController, :new
    post "/couriers/register", CourierRegistrationController, :create
    get "/couriers/log_in", CourierSessionController, :new
    post "/couriers/log_in", CourierSessionController, :create
    get "/couriers/reset_password", CourierResetPasswordController, :new
    post "/couriers/reset_password", CourierResetPasswordController, :create
    get "/couriers/reset_password/:token", CourierResetPasswordController, :edit
    put "/couriers/reset_password/:token", CourierResetPasswordController, :update
  end

  scope "/", VoltWeb do
    pipe_through [:browser, :require_authenticated_courier]

    get "/couriers/settings", CourierSettingsController, :edit
    put "/couriers/settings", CourierSettingsController, :update
    get "/couriers/settings/confirm_email/:token", CourierSettingsController, :confirm_email
    get "/couriers/dashboard", CourierDashboardController, :index
    put "/couriers/dashboard", CourierDashboardController, :update
    put "/couriers/dashboard/:id/accept", CourierDashboardController, :accept
    put "/couriers/dashboard/:id/close", CourierDashboardController, :close_delivery
  end

  scope "/", VoltWeb do
    pipe_through [:browser]

    delete "/couriers/log_out", CourierSessionController, :delete
    get "/couriers/confirm", CourierConfirmationController, :new
    post "/couriers/confirm", CourierConfirmationController, :create
    get "/couriers/confirm/:token", CourierConfirmationController, :edit
    post "/couriers/confirm/:token", CourierConfirmationController, :update
  end

  ## Authentication routes

  scope "/", VoltWeb do
    pipe_through [:browser, :redirect_if_restaurant_is_authenticated]

    get "/restaurants/register", RestaurantRegistrationController, :new
    post "/restaurants/register", RestaurantRegistrationController, :create
    get "/restaurants/log_in", RestaurantSessionController, :new
    post "/restaurants/log_in", RestaurantSessionController, :create
    get "/restaurants/reset_password", RestaurantResetPasswordController, :new
    post "/restaurants/reset_password", RestaurantResetPasswordController, :create
    get "/restaurants/reset_password/:token", RestaurantResetPasswordController, :edit
    put "/restaurants/reset_password/:token", RestaurantResetPasswordController, :update
  end

  scope "/", VoltWeb do
    pipe_through [:browser, :require_authenticated_restaurant]

    get "/restaurants/settings", RestaurantSettingsController, :edit
    put "/restaurants/settings", RestaurantSettingsController, :update
    get "/restaurants/settings/confirm_email/:token", RestaurantSettingsController, :confirm_email

    get "/restaurants/dashboard", RestaurantDashboardController, :index
    get "/restaurant/dashboard/order/:id/accept", RestaurantDashboardController, :show_accept
    put "/restaurant/dashboard/order/:id/accept", RestaurantDashboardController, :accept
    get "/restaurant/dashboard/order/:id/cancel", RestaurantDashboardController, :show_cancel
    put "/restaurant/dashboard/order/:id/cancel", RestaurantDashboardController, :cancel
    put "/restaurant/dashboard/order/:id/prepared", RestaurantDashboardController, :prepared

    get "/restaurant/dashboard/order/history", RestaurantDashboardController, :history
    get "/restaurant/dashboard/order/:id/info", RestaurantDashboardController, :prepared_info
    get "/restaurant/dashboard/order/:id/manage", RestaurantDashboardController, :show_manage

    get "/restaurants/additem", RestaurantItemController, :new
    post "/restaurants/additem", RestaurantItemController, :create
    get "/restaurants/menu", RestaurantItemController, :index
    get "/restaurants/item/:id", RestaurantItemController, :show
    get "/restaurants/edit_item/:id", RestaurantItemController, :edit
    put "/restaurants/edit_item/:id", RestaurantItemController, :update
    delete "/restaurants/delete_item/:id", RestaurantItemController, :delete
  end

  scope "/", VoltWeb do
    pipe_through [:browser]

    delete "/restaurants/log_out", RestaurantSessionController, :delete
    get "/restaurants/confirm", RestaurantConfirmationController, :new
    post "/restaurants/confirm", RestaurantConfirmationController, :create
    get "/restaurants/confirm/:token", RestaurantConfirmationController, :edit
    post "/restaurants/confirm/:token", RestaurantConfirmationController, :update
  end

  ## Authentication routes

  scope "/", VoltWeb do
    pipe_through [:browser, :redirect_if_customer_is_authenticated]

    get "/customers/register", CustomerRegistrationController, :new
    post "/customers/register", CustomerRegistrationController, :create
    get "/customers/log_in", CustomerSessionController, :new
    post "/customers/log_in", CustomerSessionController, :create
    get "/customers/reset_password", CustomerResetPasswordController, :new
    post "/customers/reset_password", CustomerResetPasswordController, :create
    get "/customers/reset_password/:token", CustomerResetPasswordController, :edit
    put "/customers/reset_password/:token", CustomerResetPasswordController, :update

  end

  scope "/", VoltWeb do
    pipe_through [:browser, :require_authenticated_customer]

    get "/customers/settings", CustomerSettingsController, :edit
    put "/customers/settings", CustomerSettingsController, :update
    get "/customers/settings/confirm_email/:token", CustomerSettingsController, :confirm_email
    get "/customers/dashboard", CustomerDashboardController, :index
    get "/customers/dashboard/:id", CustomerDashboardController, :show
    get "/customers/dashboard/item/:id", CustomerDashboardController, :order
    get "/customers/dashboard/basket/:id", CustomerDashboardController, :basket
    delete "/customers/dashboard/basket/:basket_id/delete_item/:item_id", CustomerDashboardController, :delete
    post "/customers/dashboard/item/:id", CustomerDashboardController, :add_to_basket
    post "/customers/dashboard/basket/:id", CustomerDashboardController, :submit
    put "/customers/dashboard", CustomerDashboardController, :update
    get "/customers/deposit", CustomerBalanceController, :index
    put "/customers/deposit", CustomerBalanceController, :submit
    get "/customers/history", CustomerHistoryController, :index
  end

  scope "/", VoltWeb do
    pipe_through [:browser]

    delete "/customers/log_out", CustomerSessionController, :delete
    get "/customers/confirm", CustomerConfirmationController, :new
    post "/customers/confirm", CustomerConfirmationController, :create
    get "/customers/confirm/:token", CustomerConfirmationController, :edit
    post "/customers/confirm/:token", CustomerConfirmationController, :update
  end
end
