defmodule Volt.AccountsFixtures do
  @moduledoc """
  This module defines test helpers for creating
  entities via the `Volt.Accounts` context.
  """

  def valid_phone_number, do: "+#{Enum.random(1000000000..1000000000000)}"
  def unique_courier_email, do: "courier#{System.unique_integer()}@example.com"
  def valid_courier_password, do: "hello world!"

  def valid_courier_attributes(attrs \\ %{}) do
    Enum.into(attrs, %{
      email: unique_courier_email(),
      password: valid_courier_password(),
      courier_status: "WAITING",
      first_name: "some first_name",
      last_name: "some last_name",
      phone_number: valid_phone_number()
    })
  end

  def courier_fixture(attrs \\ %{}) do
    {:ok, user} =
      attrs
      |> valid_courier_attributes()
      |> Volt.Accounts.register_courier()

    user
  end

  def extract_courier_token(fun) do
    {:ok, captured_email} = fun.(&"[TOKEN]#{&1}[TOKEN]")
    [_, token | _] = String.split(captured_email.text_body, "[TOKEN]")
    token
  end

  def unique_restaurant_email, do: "restaurant#{System.unique_integer()}@example.com"
  def valid_restaurant_password, do: "hello world!"

  def valid_restaurant_attributes(attrs \\ %{}) do
    Enum.into(attrs, %{
      address: "some address",
      closing_time: ~T[14:00:00],
      email: unique_restaurant_email(),
      first_name: "some first_name",
      last_name: "some last_name",
      name: "some name",
      opening_time: ~T[14:00:00],
      password: valid_restaurant_password(),
      phone_number: valid_phone_number(),
      city: "some city",
      zip_code: "55555",
      price_level: 42
    })
  end

  def restaurant_fixture(attrs \\ %{}) do
    {:ok, user} =
      attrs
      |> valid_restaurant_attributes()
      |> Volt.Accounts.register_restaurant()

    user
  end

  def extract_restaurant_token(fun) do
    {:ok, captured_email} = fun.(&"[TOKEN]#{&1}[TOKEN]")
    [_, token | _] = String.split(captured_email.text_body, "[TOKEN]")
    token
  end

  def unique_customer_email, do: "customer#{System.unique_integer()}@example.com"
  def valid_customer_password, do: "hello world!"

  def valid_customer_attributes(attrs \\ %{}) do
    Enum.into(attrs, %{
      address: "some address",
      balance: 120.5,
      birth_date: ~D[2022-11-04],
      card_number: "123123123123123",
      email: unique_customer_email(),
      password: valid_customer_password(),
      first_name: "some first_name",
      last_name: "some last_name",
      phone_number: valid_phone_number(),
      city: "some city",
      zip_code: "55555"
    })
  end

  def customer_fixture(attrs \\ %{}) do
    {:ok, user} =
      attrs
      |> valid_customer_attributes()
      |> Volt.Accounts.register_customer()

    user
  end

  def extract_customer_token(fun) do
    {:ok, captured_email} = fun.(&"[TOKEN]#{&1}[TOKEN]")
    [_, token | _] = String.split(captured_email.text_body, "[TOKEN]")
    token
  end
end
