defmodule Volt.SalesTest do
  use Volt.DataCase

  alias Volt.Sales

  describe "items" do
    alias Volt.Sales.Item

    import Volt.SalesFixtures

    @invalid_attrs %{description: nil, name: nil, unit_price: nil}

    test "list_items/0 returns all items" do
      item = item_fixture()
      assert Sales.list_items() == [item]
    end

    test "get_item!/1 returns the item with given id" do
      item = item_fixture()
      assert Sales.get_item!(item.id) == item
    end

    test "create_item/1 with valid data creates a item" do
      valid_attrs = %{description: "some description", name: "some name", unit_price: "some unit_price"}

      assert {:ok, %Item{} = item} = Sales.create_item(valid_attrs)
      assert item.description == "some description"
      assert item.name == "some name"
      assert item.unit_price == "some unit_price"
    end

    test "create_item/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = Sales.create_item(@invalid_attrs)
    end

    test "update_item/2 with valid data updates the item" do
      item = item_fixture()
      update_attrs = %{description: "some updated description", name: "some updated name", unit_price: "some updated unit_price"}

      assert {:ok, %Item{} = item} = Sales.update_item(item, update_attrs)
      assert item.description == "some updated description"
      assert item.name == "some updated name"
      assert item.unit_price == "some updated unit_price"
    end

    test "update_item/2 with invalid data returns error changeset" do
      item = item_fixture()
      assert {:error, %Ecto.Changeset{}} = Sales.update_item(item, @invalid_attrs)
      assert item == Sales.get_item!(item.id)
    end

    test "delete_item/1 deletes the item" do
      item = item_fixture()
      update_attrs = %{orderable: false}

      assert {:ok, %Item{} = item} = Sales.update_item(item, update_attrs)
      assert item.orderable == false
    end

    test "change_item/1 returns a item changeset" do
      item = item_fixture()
      assert %Ecto.Changeset{} = Sales.change_item(item)
    end
  end
end
