defmodule VoltWeb.CustomerRegistrationControllerTest do
  use VoltWeb.ConnCase, async: true

  import Volt.AccountsFixtures

  describe "GET /customers/register" do
    test "renders registration page", %{conn: conn} do
      conn = get(conn, Routes.customer_registration_path(conn, :new))
      response = html_response(conn, 200)
      assert response =~ "<h1>Register</h1>"
      assert response =~ "Log in</a>"
      assert response =~ "<div>\n<button id=\"submit_button\" type=\"submit\">Register</button>"
    end

    test "renders location map", %{conn: conn} do
      conn = get(conn, Routes.customer_registration_path(conn, :new))
      response = html_response(conn, 200)
      assert response =~ "id=\"myMap\""
      assert response =~ "Microsoft.Map"
    end

    test "redirects if already logged in", %{conn: conn} do
      conn = conn |> log_in_customer(customer_fixture()) |> get(Routes.customer_registration_path(conn, :new))
      assert redirected_to(conn) == "/customers/dashboard"
    end
  end

  describe "POST /customers/register" do
    @tag :capture_log
    test "creates account and logs the customer in", %{conn: conn} do
      email = unique_customer_email()

      conn =
        post(conn, Routes.customer_registration_path(conn, :create), %{
          "customer" => valid_customer_attributes(email: email)
        })

      assert get_session(conn, :customer_token)
      assert redirected_to(conn) == "/customers/dashboard"

      # Now do a logged in request and assert on the menu
      conn = get(conn, "/")
      response = html_response(conn, 200)
      assert response =~ "Settings</a>"
      assert response =~ "Log out</a>"
    end

    test "render errors for invalid data", %{conn: conn} do
      conn =
        post(conn, Routes.customer_registration_path(conn, :create), %{
          "customer" => %{"email" => "with spaces", "password" => "too short"}
        })

      response = html_response(conn, 200)
      assert response =~ "<h1>Register</h1>"
      assert response =~ "must have the @ sign and no spaces"
      assert response =~ "should be at least 12 character"
    end
  end
end
