defmodule VoltWeb.RestaurantRegistrationControllerTest do
  use VoltWeb.ConnCase, async: true

  import Volt.AccountsFixtures

  describe "GET /restaurants/register" do
    test "renders registration page", %{conn: conn} do
      conn = get(conn, Routes.restaurant_registration_path(conn, :new))
      response = html_response(conn, 200)
      assert response =~ "<h1>Register</h1>"
      assert response =~ "Log in</a>"
      assert response =~ "<div>\n<button id=\"submit\" type=\"submit\">Register</button>"
    end

    test "redirects if already logged in", %{conn: conn} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      assert redirected_to(conn) == "/restaurants/dashboard"
    end
  end

  describe "POST /restaurants/register" do
    @tag :capture_log
    test "creates account and logs the restaurant in", %{conn: conn} do
      email = unique_restaurant_email()

      conn =
        post(conn, Routes.restaurant_registration_path(conn, :create), %{
          "restaurant" => valid_restaurant_attributes(email: email)
        })

      assert get_session(conn, :restaurant_token)
      assert redirected_to(conn) == "/restaurants/dashboard"

      # Now do a logged in request and assert on the menu
      conn = get(conn, "/")
      response = html_response(conn, 200)
      assert response =~ email
      assert response =~ "Settings</a>"
      assert response =~ "Log out</a>"
    end

    test "render errors for invalid data", %{conn: conn} do
      conn =
        post(conn, Routes.restaurant_registration_path(conn, :create), %{
          "restaurant" => %{"email" => "with spaces", "password" => "too short"}
        })

      response = html_response(conn, 200)
      assert response =~ "<h1>Register</h1>"
      assert response =~ "must have the @ sign and no spaces"
      assert response =~ "should be at least 12 character"
    end
  end
end
